/*
 * LICENCE[[
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1/CeCILL 2.O 
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 * 
 * The Original Code is utc.fr code. 
 * 
 * The Initial Developer of the Original Code is 
 * david.rivron@utc.fr
 * 
 * Portions created by the Initial Developer are Copyright (C) 2012
 * the Initial Developer. All Rights Reserved. 
 * 
 * Contributor(s): 
 * 
 * 
 * Alternatively, the contents of this file may be used under the terms of 
 * either of the GNU General Public License Version 2 or later (the "GPL"), 
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"), 
 * or the CeCILL Licence Version 2.0 (http://www.cecill.info/licences.en.html), 
 * in which case the provisions of the GPL, the LGPL or the CeCILL are applicable 
 * instead of those above. If you wish to allow use of your version of this file 
 * only under the terms of either the GPL or the LGPL, and not to allow others 
 * to use your version of this file under the terms of the MPL, indicate your 
 * decision by deleting the provisions above and replace them with the notice 
 * and other provisions required by the GPL or the LGPL. If you do not delete 
 * the provisions above, a recipient may use your version of this file under 
 * the terms of any one of the MPL, the GPL, the LGPL or the CeCILL.
 * ]]LICENCE
 */
 
 var MessagesHandler = require('./MessagesHandler.js'),
	util = require('util');
/**
 * Handler d'un message de type 'uri'.
 * 
 * Propriétés du message client : {
 * 	type: uri,
 *		account: <utilisateur à l'origine de l'action> 
 * 	uri: <URI concernée par l'action>
 * 	action: focus | unfocus| dirty | save | close,
 * }
 * 
 * Propriétés du message serveur : {
 * 	type: uri,
 * 	account:  <utilisateur à l'origine du changement d'état> 
 * 	uri: <URI concernée par le changement d'état>
 * 	states: <états de l'URI pour l'utilisateur>
 * }
 */
	
var UriMessagesHandler = module.exports = function (pCollabServer) {
	MessagesHandler.call(this, 'uri', pCollabServer);
}

util.inherits(UriMessagesHandler, MessagesHandler);

UriMessagesHandler.prototype.onInit = function (pClient, pMessage) {
	pClient.urisStates = pMessage.urisStates;
	for (var vUri in pClient.urisStates) {
		var vStates = pClient.urisStates[vUri];
		var vEvents = [];
		if (vStates.reading) vEvents.push('focus');
		if (vStates.writing) vEvents.push('dirty');
		for (var i=0; i<vEvents.length; i++) {
			this.xDispatchUriMessage(vUri, pClient, vEvents[i]);
		}
	}
};
	
UriMessagesHandler.prototype.onMessage = function (pClient, pMessage) {
	var vUri = pMessage.uri;
	var vClients = this.server.clients;
	var vWspCd = vUri.substr(0, vUri.indexOf('/'));
	if (!this.requirePerm(pClient, vWspCd, pMessage)) return;
	var vEvent = pMessage.action;

	if (pMessage.action == 'requestStates') {
		// Envoie les états des autres utilisateurs pour cette URI
		for (var vClientId in this.server.clients) {
 			var vClient = vClients[vClientId];
 			if (vClient != pClient && vUri in vClient.urisStates && vClient.urisStates[vUri]) {
 				var vServerMessage = { type: 'uri', uri: vUri, states: vClient.urisStates[vUri], account: vClient.account, clientId: vClient.clientId };
 				console.info('->', pClient.account, vServerMessage);
 				pClient.socket.emit('message', vServerMessage);
 			}
 		}
	} else {
		vStates = pClient.urisStates[vUri];
		if (!vStates) vStates = pClient.urisStates[vUri] = { opened: false, reading: false, writing: false };
		var vOldMergedStates = this.xMergeUriStates(vUri, pMessage.account);

		switch (pMessage.action) {
	 	case 'open':
	 		vStates.opened = true;
	 	case 'focus':
			vStates.reading = true;
			break;
		case 'unfocus':
			vStates.reading = false;
			break;
		case 'dirty':
			var vIsLocked = Object.keys(vClients).some(function(pCliId) {
				var vUriStates = vClients[pCliId].urisStates[vUri];
				return vUriStates && vUriStates.writing;
			});
			if (vIsLocked) {
				console.error('Write conflict on ', vUri);
				return;
			}
			vStates.writing = true;
			break;
		case 'save':
			vStates.writing = false;
			break;
		case 'close':
			for (var vState in vStates) vStates[vState] = false;
		}
		
		var vEvents = [];
		var vNewMergedStates = this.xMergeUriStates(vUri, pMessage.account);
		if (vNewMergedStates && vNewMergedStates && vNewMergedStates.writing != vOldMergedStates.writing) {
			if (vNewMergedStates.writing) vEvents.push('dirty');
			else vEvents.push('undirty');
		}
		if (vNewMergedStates.reading != vOldMergedStates.reading) {
			if (vNewMergedStates.reading) vEvents.push('focus');
			else vEvents.push('unfocus');
		}
		if (vNewMergedStates.opened != vOldMergedStates.opened) {
			if (vNewMergedStates.opened) vEvents.push('open');
			else vEvents.push('close');
		}
		console.info(vOldMergedStates, vNewMergedStates, vEvents);
		
		this.xDispatchUriMessage(vUri, pClient, vEvents);

		var vHasState = Object.keys(vStates).some(function(pState) {
			return vStates[pState];
		});
		if (!vHasState) delete pClient.urisStates[vUri];
	}
}


UriMessagesHandler.prototype.xMergeUriStates = function (pUri, pAccount) {
	var vMergedStates = {};
	
	for (var vClientId in this.server.clients) {
		var vClient = this.server.clients[vClientId];
		if (pUri in vClient.urisStates && vClient.account == pAccount) {
			var vStates = vClient.urisStates[pUri];
			for (var vState in vStates) {
				vMergedStates[vState] = vMergedStates[vState] || vStates[vState];
			}
		}
	}
	return vMergedStates;
}

UriMessagesHandler.prototype.onClose = function (pClient) { 
	for (var vUri in pClient.urisStates) {
		this.onMessage(pClient, { type: 'uri', uri: vUri, action: 'close', account: pClient.account, clientId: pClient.clientId });
	}
}

UriMessagesHandler.prototype.xDispatchUriMessage = function  (pUri, pClient, pEvent, pStates) {
	// Envoie du changement d'état aux client concernés par le workspace de l'URI
	var vWspCd = pUri.substr(0, pUri.indexOf('/'));
	var vMessage = { type: 'uri', uri: pUri, account: pClient.account, event: pEvent, states: pStates, clientId: pClient.clientId };
	for (var vClientId in this.server.clients) {
		var vClient = this.server.clients[vClientId];
		if (vClient != pClient && vClient.wsps && vClient.wsps.indexOf(vWspCd) != -1) {
			this.server.logMessage(vMessage, vClient.account);
			vClient.socket.emit('message', vMessage);
		}
	}
}


